#include <string.h>

/*
	todo:  
*/

#include "disp.h"
#include "mcp23s18.h"

#ifdef __MT12864J__ 

#include "MDR32F9Qx_port.h"
#include "MDR32F9Qx_rst_clk.h"

#include "itmr.h"
#include "misc.h"
#include "mt12864j_drv.h"
#include "mt12864j_font.h"

#define MT12864J_ON		0x3F
#define MT12864J_OFF		0x3E 


static void mt12864_HardwareReset(void)
{ 
	mcp23s18_WriteBit(LCD_RES,0);
	itmr_Delay(10);
	
	mcp23s18_WriteBit(LCD_RES,1);
	itmr_Delay(1000);
	
}

static void mt12864j_OnOffCtrl(unsigned char cmd)
{ unsigned short port_state = 0;
	
	port_state |= (cmd ? (MT12864J_ON) : (MT12864J_OFF));
	mcp23s18_StartLCD(0,2,port_state);
	
	
}

static void mt12864j_SetStartLine(unsigned char line_num)
{
	unsigned short port_state = 0;
	
	port_state |= (0xC0 | (line_num & 0x3F));
	mcp23s18_StartLCD(0,2,port_state);
	
}

static void mt12864j_SetPage(unsigned char crystal, unsigned char page_num)
{
	unsigned short port_state = 0;
	
	port_state |= (0xB8 | (page_num & 0x7));
	mcp23s18_StartLCD(0,crystal,port_state);
	
}

static void mt12864j_SetAddr(unsigned char addr)
{
	unsigned short port_state = 0;
	unsigned char crystal = addr < 64 ? 0 : 1;
	
	port_state |= (0x40 | (addr & 0x3F));
	mcp23s18_StartLCD(0,crystal,port_state);
	
}

static void mt12864j_WriteData(unsigned char data, unsigned int addr)
{
	if(addr < 128){
//			unsigned short port_state = 0;
			unsigned char crystal = addr < 64 ? 0 : 1;
			
			mt12864j_SetAddr(addr);				
		  
			mcp23s18_StartLCD(1,crystal,data);		
			
	}
}

//------------------------------------------------------------
static void mt12864j_HardwareInit(void)
{ 
	  mt12864_HardwareReset();
	 
}

//------------------------------------------------------------
void mt12864j_SetEqData(unsigned char data)
{
	unsigned char page,addr;
	
	mt12864j_SetAddr(0);
	for(page = 0; page < 8; page++){
		mt12864j_SetPage(0, page);
		for(addr = 0; addr < 64; addr++){
			mt12864j_WriteData(data & 0xFF, addr);			
		}
	}		

	mt12864j_SetAddr(64);
	for(page = 0; page < 8; page++){
		mt12864j_SetPage(1, page);
		for(addr = 64; addr < 128; addr++)
			mt12864j_WriteData(data & 0xFF, addr);
	}		
	mt12864j_SetAddr(0);
	mt12864j_SetPage(0, 0);
	mt12864j_SetAddr(64);
	mt12864j_SetPage(1, 0);
}

//------------------------------------------------------------
void mt12864j_SetEqDataToPage(unsigned char crystal, unsigned char page, unsigned char data)
{
	unsigned char addr;
	
	mt12864j_SetPage(crystal, page);	
	
	if(!crystal)	
		mt12864j_SetAddr(0);						
	else 			
		mt12864j_SetAddr(64);

	for(addr = 0; addr < 64; addr++)
		mt12864j_WriteData(data, addr);	
}

//------------------------------------------------------------
void mt12864j_SetEqDataToPageAddr(unsigned char page, unsigned char start_addr, unsigned char stop_addr, unsigned char data)
{
	unsigned char addr;

	if(start_addr > stop_addr) return;

	if((start_addr < 64) && (stop_addr < 64))
		mt12864j_SetPage(0, page);
	else{
		if((start_addr >= 64) && (stop_addr >= 64))
			mt12864j_SetPage(1, page);			
		else{
			mt12864j_SetPage(0, page);						
			mt12864j_SetPage(1, page);						
		}
	}	
	mt12864j_SetAddr(start_addr);	
			
	for(addr = start_addr; addr < stop_addr; addr++)
		mt12864j_WriteData(data, addr);										
}

//------------------------------------------------------------
void mt12864j_PrintChar(unsigned char page, unsigned char addr, unsigned char data, unsigned char show_mode)
{
	unsigned char cnt;
	unsigned int off = data * 6;
	
	mt12864j_SetPage(0, page);
	mt12864j_SetPage(1, page);
	mt12864j_SetAddr(addr);

	if(show_mode){
		for(cnt = 0; cnt < 6;)
			mt12864j_WriteData(~mt12864j_font[off + cnt++], addr++);	
	}	
	else{			
		for(cnt = 0; cnt < 6;)
			mt12864j_WriteData(mt12864j_font[off + cnt++], addr++);	
	}
}

//------------------------------------------------------------
void mt12864j_PrintString(unsigned char page, unsigned char addr, unsigned char* string, unsigned char show_mode)
{
	unsigned int str_ind, str_length;
		
	if((str_length = strlen((char*)string)) > 21)
		str_length = 21;	
	
	mt12864j_ClearPageAddr(page, addr, addr + str_length * 6 + 6);

	if(addr < 64){
		mt12864j_SetAddr(addr);
		mt12864j_SetAddr(64);
	}
	else{
		mt12864j_SetAddr(0);
		mt12864j_SetAddr(addr);		
	}
	
	mt12864j_SetPage(0, page);	
	mt12864j_SetPage(1, page);

	for(str_ind = 0; str_ind < str_length; str_ind++, addr += 6)
		mt12864j_PrintChar(page, addr, string[str_ind], show_mode);	 
}

//------------------------------------------------------------
void mt12864j_PrintNum(unsigned char page, unsigned char addr, unsigned int val, unsigned char show_mode)
{	
	#define MAX_NUM_CHAR_CNT	11
	unsigned char temp[MAX_NUM_CHAR_CNT];	
	unsigned int cnt;	
		
	cnt = uint2text(val, temp, MAX_NUM_CHAR_CNT - 1);
	temp[MAX_NUM_CHAR_CNT - 1] = 0;
	memcpy(temp, temp + MAX_NUM_CHAR_CNT - cnt - 1, (cnt + 1) * sizeof(unsigned char));
											
	mt12864j_PrintString(page, addr, temp, show_mode);	
}

//------------------------------------------------------------
void mt12864j_Point(unsigned int x, unsigned int y)
{
	unsigned char page = y / 8;

	mt12864j_SetPage(0, page);
	mt12864j_SetPage(1, page);	
			
	mt12864j_WriteData(1 << (y - 8 * page),x);
}

//------------------------------------------------------------
void mt12864j_Byte(unsigned char page, unsigned char addr, unsigned char val)
{
	mt12864j_SetPage(0, page);
	mt12864j_SetPage(1, page);			

	mt12864j_WriteData(~val, addr);
}

void mt12864j_Init(void)
{
	mt12864j_HardwareInit();
	mt12864j_OnOffCtrl(1);
	mt12864j_Clear();	
	mt12864j_SetStartLine(0);
}


#endif
